#!/usr/bin/env python3
import sys
import os
import tempfile

# [+] Config
verbose_output = False
colorful_output = False

if verbose_output and colorful_output:
  import colorama

def main():
  my_dir = os.path.dirname(os.path.realpath(__file__))
  tempdir = tempfile.mkdtemp(dir = my_dir)

  success_modes = []
  failure_modes = []

  for mode in range(0o000, 0o1000):
    try:
      res = os.open(os.path.join(tempdir, str(mode)),
        os.O_CREAT | os.O_EXCL | os.O_RDWR,
        mode)
      success_modes.append(mode)
    except PermissionError:
      failure_modes.append(mode)

  print("Successes:")
  print(list(map(oct, success_modes)))

  print("Failures:")
  print(list(map(oct, failure_modes)))

  if verbose_output:
    for mode in range(0o000, 0o1000):
      buf = []

      if colorful_output:
        if mode in success_modes:
          buf.append(colorama.Fore.GREEN)
        elif mode in failure_modes:
          buf.append(colorama.Fore.RED)
        else:
          buf.append(colorama.Fore.BLUE)

      buf.append(str(oct(mode)))
      buf.append("\t\t{}".format(perm_to_text(mode)))

      if mode in success_modes:
        buf.append("\t PASS")
      elif mode in failure_modes:
        buf.append("\t\t FAIL")
      else:
        buf.append("\t\t\tWTF")

      if colorful_output:
        buf.append(colorama.Style.RESET_ALL)

      print("".join(buf))

def perm_to_text(perm):
  res = []
  if perm < 0 or perm > 0o777:
    raise ValueError
  # For each nibble
  for i in range(2,-1,-1):
    # Extract the nibble
    nibble = (perm & (0o7 << (i*3))) >> (i*3)
    # Test the nibble for rwx
    res.append("r" if nibble & 0o4 else "-")
    res.append("w" if nibble & 0o2 else "-")
    res.append("x" if nibble & 0o1 else "-")
  return "".join(res)

if (__name__ == "__main__"):
  sys.exit(main())
